%---------------------------- ZIMLOAD ---------------------------------
%
%    Purpose: ZIMLOAD is used to "bootstrap" objects into a database.
%             Object definitions (and data) are loaded from structured
%             documents which are typically built by ZIMSAVE.
%
%    Syntax:  ZIMLOAD [? | Z50 STATIC BATCH [NOCREATE]]
%
%    Notes:   1) The structured documents for the Object Dictionary
%             information are pre-defined in the ZIM Utilities directory,
%             $Utils.
%
%             2) BATCH operation does not interact with the user.
%
%             3) NOCREATE only applies if BATCH is selected.
%
%             4) Object definitions existing in the Object Dictionary
%             before ZIMLOAD executes are not affected.
%
%             5) Under normal operation, ZIMLOAD accesses and releases
%             the necessary Zim Directories as each new object is created.
%             This is done to avoid exhausting operating system limits on
%             the number of files that can be open at any one time.  If this
%             is not a concern, you can have ZIMLOAD access each Zim directory
%             once at the start and leave the directories accessed throughout
%             the ZIMLOAD process by using the STATIC option.
%
%----------------------------------------------------------------------

%----------------------------------------------------------------------
%% Local Procedures

%----------------------------------------------------------------------
%
% plBreakHandler
%    General break handler
%
%----------------------------------------------------------------------
localprocedure plBreakHandler()
   set messages off
   if "#<UtilsAlreadyAcc>" = $false
      release $utils
   endif
   SET RESTORE
endprocedure

%---------------------------- UsageError -------------------------------
localprocedure UsageError()

output ""
output "Syntax:  ZIMLOAD Z50 STATIC BATCH [NOCREATE]"
output ""
output "   where:"
output "         Z50      indicates object definitions were produced by "
output "                  Zim version 8. "
output ""
output "         STATIC   indicates Zim directories are to be accessed once"
output "                  only at the start of the loading process, rather than"
output "                  for each object encountered."
output ""
output "         BATCH    indicates ZIMLOAD will operate without pausing for"
output "                  user confirmation before proceeding to the next step."
output ""
output "         NOCREATE indicates ZIMLOAD will not create any of the objects"
output "                  whose definitions are loaded.  NOCREATE can only be"
output "                  used with BATCH."
output ""
   let <BreakOut> = "Yes"
endprocedure

%----------------------------------------------------------------------
%
% plParseCommandLine
%    Parse the command line.
%
%----------------------------------------------------------------------
localprocedure plParseCommandLine (arg1, arg2, arg3, arg4, arg5)

   if arg1 = '?'
      UsageError()
      RETURN
   endif

%% Establish default processing settings.
   let <LoadFormat> = "Z50" \
       <UIMode> = "PROMPT"  \
       <CrtMode> = "CREATE"  $utils._diraccess = "DYNAMIC"

%% Establish the format of the definition data to load.
   if arg1 = 'Z50'
      let <LoadFormat> = arg1
      let arg1=arg2 arg2=arg3 arg3=arg4 arg4=arg5      % Shift arguments.
   endif

%% Static or dynamic Zim directory access during loading.
   if arg1 = 'STATIC'
      let $utils._diraccess = "STATIC"
      let arg1=arg2 arg2=arg3 arg3=arg4               % Shift arguments.
   endif

%% Interactive or batch operation.  If batch, create objects or not.
   if arg1 = 'BATCH'
      let <UIMode> = "BATCH"
      let arg1=arg2 arg2=arg3                         % Shift arguments.
      if arg1 = 'NOCREATE'
         let <CrtMode> = "NOCREATE"
         let arg1=arg2                                % Shift arguments.
      endif
   endif

   if not arg1 = ''
      UsageError()
      RETURN
   endif

endprocedure

%----------------------------------------------------------------------
%
% plAccessUtilDir
%    Access $utils directory
%
%----------------------------------------------------------------------
localprocedure plAccessUtilDir()

on error
   let <9> = $MessageBox($concat( \
                 'Unable to access Zim utilities directory, "$Utils".', \
                 '\0dCheck that your ZIM environment variable is set'), \
                 cErrorMsg, cOKStyle, cDefaultButton1, "ZimLoad") \
       <BreakOut> = "Yes"
   RETURN
endon

   if $dirpath() like '%;$Utils(%'
      let <UtilsAlreadyAcc> = $true
   else
      set errors off
      set warnings off
      access $Utils read
   endif

   let $utils._dirstring = ""

endprocedure

%----------------------------------------------------------------------
%
% plChkUserDefs
%    Updates Users entity set entries for ZIM and super using following rules:
%    1. If there is a ZIM(super) entry in both Users and _datusers,
%       update current record with what is in _datusers.
%    2. If there is no ZIM(super) entry in _datusers, delete coresponding
%       entry in Users.
%    3. If, after the above, there is no ZIM(super) entry in Users, add one.
%
%----------------------------------------------------------------------
localprocedure plChkUserDefs( )

   SET SAVE
   set messages off

   %% handle any changes made to userid's and groupid's
   %% also, if user didn't have ZIM or super, delete them
%%   change users where Username="ZIM" \
%%      from $utils._datusers where $utils._datusers.UserName = "ZIM"
%%   change users where Username="super" \
%%      from $utils._datusers where $utils._datusers.UserName = "super"
%%   find $utils._datusers where UserName="ZIM"
%%   if $setcount = 0
     delete Users where UserName="ZIM"
%%   endif
%%   find $utils._datusers where UserName="super"
%%   if $setcount = 0
     delete Users where UserName="super"
%%   endif

   % if users document wasn't present, or ZIM and super weren't added,
   % restore them to the user entity set. Set messages off because, if
   % they are there already, they cannot be added again.
   add users let username = "ZIM" userid = 0 groupid = 0
   add users let username = "super" userid = 0 groupid = 0

   SET RESTORE
endprocedure

%----------------------------------------------------------------------
%
% plLoadDDDefs1
%   Load up the Object Dictionary.
%
%----------------------------------------------------------------------
localprocedure plLoadDDDefs1( vlFormat )

   if $iswinopen('wDevCen') = $true
      output ''
   else
      window clear
   endif
   output "************************* ZIMLOAD **************************"
   output

   SET SAVE
   set messages off
   find 0 Documents     -> $docset
   find 0 Variables     -> $varset
   find 0 EntitySets    -> $entset
   find 0 Relationships -> $relset
   find 0 Roles         -> $roleset
   find 0 Forms         -> $formset
   find 0 Displays      -> $dispset
   find 0 Windows       -> $winset
   find 0 Directories   -> $dirset
   find 0 NamedSets     -> $setset
   find 0 Constants     -> $constset
   find 0 Menus         -> $menuset

   if vlFormat = "Z50"
      %% Delete existing stuff for ZIM and super.
      %% If they are imported fine. If not standard ones are added later.
      delete all Users where UserName in ("ZIM", "super")

      %% The second parameter tells ZOM load routines to create
      %% sets of what is added. These are used during create phase.
      pZOMImportOD(vlFormat, $true)

      % if users document wasn't present, or ZIM and super weren't added,
      % restore them to the user entity set. Set messages off because, if
      % they are there already, they cannot be added again.
      set save
      set messages off
      add users let username = "ZIM" userid = 0 groupid = 0
      add users let username = "super" userid = 0 groupid = 0
      set restore
   endif

   SET RESTORE
endprocedure

%----------------------------------------------------------------------
%
% plCrtObjs1
%   Now the Object Dictionary is loaded, create the objects. The loading
%   created some named sets. We loop through each doing a CREATE. Special
%   care must be done for roles and relationships since there may be some
%   creation interdependencies.
%
%----------------------------------------------------------------------
localprocedure plCrtObjs1() \
   local (vlResponse)

   if $iswinopen('wDevCen') = $true
      output ''
   else
      window clear
   endif
   output "************************* ZIMLOAD **************************"
   output

   %% In a series of "while" loops we create:
   %% Documents, Entity Sets, Relationships, Roles, Variables,
   %% Forms and Displays, etc.

   SET SAVE
   set reset

   output "CREATING ALL DIRECTORIES"
%%   let $setcount = $lastmember("$dirset")
%%   set currentset $dirset
%%   while $setcount > 0
%%      let <ObjName> = $trim(dirname)
%%      let <DirName> = {$trim(ownerdirname) where Ownerdirname <> "", "zim"}
%%      let <CreateOption> = ''
%%      _access ("#<DirName>")
%%      if $exist$('#<DirName>','#<ObjName>',9)=$false
%%         output 'CREATING DIRECTORY: #<ObjName> in #<DirName>'
%%         set messages off
%%CreateDir:
%%         create directory #<ObjName> in #<DirName> #<CreateOption>
%%         if $lasterrcode in (1036, 7866) % N.B. "On Error" trap won't work here
            %% Directory, Entity, or Relationship file already exists.
%%            set escapechar on
%%            let vlResponse = $messagebox($concat('The file "', \
%%                   $substring($lasterrmessage, \
%%                      $position($lasterrmessage, '("') + 2, 7), \
%%                   '" already exists.\0dOverwrite it?'), \
%%                   cErrorMsg, cYesNoStyle, cDefaultButton1, \
%%                   'ZimLoad - Create Directories')
%%            if vlResponse = 'YES'
%%               let <CreateOption> = 'delete'
%%               goto CreateDir
%%            endif
%%         endif
%%         set messages on
%%         if _diraccess = "STATIC"
%%            output 'ACCESSING DIRECTORY: #<ObjName> UPDATE'
%%            access #<DirName>.#<ObjName> update
%%         endif
%%      endif
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   output "CREATING ALL DOCUMENTS"
%%   let $setcount = $lastmember("$docset")
%%   set currentset $docset
%%   while $setcount > 0
%%      let <ObjName> = $trim(docname)
%%      let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%      _access ("#<DirName>")
%%      if $exist$('#<DirName>','#<ObjName>',9)=$false
%%         output 'CREATING DOCUMENT: #<ObjName> in #<DirName>'
%%         create document #<ObjName> in #<DirName>
%%      endif
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   output "CREATING ALL VARIABLES"
%%   let $setcount = $lastmember("$varset")
%%   set currentset $varset
%%   while $setcount > 0
%%      let <ObjName> = $trim(varname)
%%      let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%      output 'CREATING VARIABLE: #<ObjName> in #<DirName>'
%%      _access ("#<DirName>")
%%      create variable #<ObjName> in #<DirName>
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   output "CREATING ALL ENTITY SETS"
%%   let $setcount = $lastmember("$entset")
%%   set currentset $entset
%%   while $setcount > 0
%%      let <ObjName>=$trim(entname)
%%      let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%      let <CreateOption> = ''
%%      output 'CREATING ENTITY SET: #<ObjName> in #<DirName>'
%%      _access ("#<DirName>")
%%      set messages off
%%CreateEnt:
%%      create entityset #<ObjName> in #<DirName> #<CreateOption>
%%      if $lasterrcode in (1036, 7866) % N.B. "On Error" trap won't work here
         %% Directory, Entity, or Relationship file already exists.
%%         set escapechar on
%%         let vlResponse = $messagebox($concat('The file "', \
%%                $substring($lasterrmessage, \
%%                   $position($lasterrmessage, '"') + 1, 7), \
%%                '" already exists.\0dOverwrite it?'), \
%%                cErrorMsg, cYesNoStyle, cDefaultButton1, \
%%                'ZimLoad - Create Entity Sets')
%%         if vlResponse = 'YES'
%%            let <CreateOption> = 'delete'
%%            goto CreateEnt
%%         endif
%%      endif
%%      set messages on
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   output "CREATING ALL ROLES FOR ENTITY SETS"
%%   let $setcount = $lastmember("$roleset")
%%   set currentset $roleset
%%   while $setcount > 0
%%      let <ObjName>=$trim(rolename)
%%      let <OwnerName>=$trim(ownername)
%%      let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%      _access ("#<DirName>")
%%      if $exist$('#<DirName>','#<OwnerName>',4)=$true
%%          output 'CREATING ROLE' $tochar('#<ObjName>',18) \
%%             'FOR ENTITY SET' $tochar('#<OwnerName>',18) " in #<DirName>"
%%          create role #<ObjName> in #<DirName>
%%      endif
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   output "CREATING ALL FORMS"
%%   let $setcount = $lastmember("$formset")
%%   set currentset $formset
%%   while $setcount > 0
%%      let <ObjName>=$trim(formname)
%%      let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%      output 'CREATING FORM: #<ObjName> in #<DirName>'
%%      _access ("#<DirName>")
%%      create form #<ObjName> in #<DirName>
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   output "CREATING ALL MENUS"
%%   let $setcount = $lastmember("$menuset")
%%   set currentset $menuset
%%   while $setcount > 0
%%      let <ObjName>=$trim(menuname)
%%      let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%      output 'CREATING MENU: #<ObjName> in #<DirName>'
%%      _access ("#<DirName>")
%%      create menu #<ObjName> in #<DirName>
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   output "CREATING ALL RELATIONSHIPS"
%%   let $setcount = $lastmember("$relset")
%%   set currentset $relset
%%   while $setcount > 0
%%      let <ObjName>=$trim(relname)
%%      let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%      let <CreateOption> = ''
%%      output 'CREATING RELATIONSHIP: #<ObjName> in #<DirName>'
%%      _access ("#<DirName>")
%%      set messages off
%%CreateRel:
%%      create relationship #<ObjName> in #<DirName> #<CreateOption>
%%      if $lasterrcode in (1036, 7866) % N.B. "On Error" trap won't work here
         %% Directory, Entity, or Relationship file already exists.
%%         set escapechar on
%%         let vlResponse = $messagebox($concat('The file "', \
%%                $substring($lasterrmessage, \
%%                   $position($lasterrmessage, '"') + 1, 7), \
%%                '" already exists.\0dOverwrite it?'), \
%%                cErrorMsg, cYesNoStyle, cDefaultButton1, \
%%                'ZimLoad - Create Relationships')
%%         if vlResponse = 'YES'
%%            let <CreateOption> = 'delete'
%%            goto CreateRel
%%         endif
%%      endif
%%      set messages on
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   output "CREATING ALL ROLES FOR RELATIONSHIPS"
%%   let $setcount = $lastmember("$roleset")
%%   set currentset $roleset
%%   top $roleset
%%   while $setcount > 0
%%      let <ObjName>=$trim(rolename)
%%      let <OwnerName>=$trim(ownername)
%%      let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%      _access ("#<DirName>")
%%      if $exist$('#<DirName>','#<OwnerName>',6)=$true or \
%%         $exist$('#<DirName>','#<OwnerName>',7)=$true
%%         output 'CREATING ROLE' $tochar('#<ObjName>',18) \
%%            'FOR RELATIONSHIP' $tochar('#<OwnerName>',18) " in #<DirName>"
%%         create role #<ObjName> in #<DirName>
%%      endif
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   output "CREATING ALL DISPLAYS"
%%   let $setcount = $lastmember("$dispset")
%%   set currentset $dispset
%%   while $setcount > 0
%%      let <ObjName>=$trim(displayname)
%%      let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%      output 'CREATING DISPLAY: #<ObjName> in #<DirName>'
%%      _access ("#<DirName>")
%%      create disp #<ObjName> in #<DirName>
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   output "CREATING ALL WINDOWS"
%%   let $setcount = $lastmember("$winset")
%%   set currentset $winset
%%   while $setcount > 0
%%      let <ObjName>=$trim(winname)
%%      let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%      output 'CREATING WINDOW: #<ObjName> in #<DirName>'
%%      _access ("#<DirName>")
%%      create window #<ObjName> in #<DirName>
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   output "CREATING ALL NAMEDSETS"
%%   let $setcount = $lastmember("$setset")
%%   set currentset $setset
%%   while $setcount > 0
%%      let <ObjName>=$trim(setname)
%%      let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%      output 'CREATING NAMEDSET: #<ObjName> in #<DirName>'
%%      _access ("#<DirName>")
%%      create set #<ObjName> in #<DirName>
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   output "CREATING ALL CONSTANTS"
%%   let $setcount = $lastmember("$constset")
%%   set currentset $constset
%%   while $setcount > 0
%%      let <ObjName> = $trim(Constants.constname)
%%      let <DirName> = {$trim(Constants.dirname) where Constants.dirname <> "", "zim"}
%%      output 'CREATING CONSTANT: #<ObjName> in #<DirName>'
%%      _access ("#<DirName>")
%%      create constant #<ObjName> in #<DirName>
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

%%   change all constants $utils._datconstrel $utils._datconsts \
%%      where \
%%         constants.constvalue is not $null and \
%%         constants.constvalue <> $utils._datconsts.constvalue or \
%%         constants.consttype is not $null and \
%%         constants.consttype <> $utils._datconsts.consttype \
%%      let \
%%         constants.consttype = \
%%            { $utils._datconsts.consttype where $utils._datconsts.ConstType <> "-----" } \
%%         constants.constvalue = $utils._datconsts.constvalue \
%%      -> $constset2
%%   find $constset2 keep constants -> $constset

%%   while $setcount > 0
%%      let <ObjName> = $trim(Constants.constname)
%%      let <DirName> = {$trim(Constants.dirname) where Constants.dirname <> "", "zim"}
%%      output 'RECREATING CONSTANT: #<ObjName> in #<DirName>'
%%      _access ("#<DirName>")
%%      erase constant #<ObjName> from #<DirName>
%%      create constant #<ObjName> in #<DirName>
%%      _release()
%%      next
%%      let $setcount = $setcount - 1
%%   endwhile

   SET RESTORE
endprocedure

%----------------------------------------------------------------------
%
% plLoadData
%   Now, if requested, restore data to entity sets and/or relationships.
%   This data must have been saved by ZIMSAVE.
%
%----------------------------------------------------------------------
localprocedure plLoadData()

   if $iswinopen('wDevCen') = $true
      output ''
   else
      window clear
   endif
   output "************************* ZIMLOAD **************************"
   output


   SET SAVE
   set messages off
   SET EOLNTRANSLATE $Utils.cEOLNTRANSLATE

%%   if $lastmember ("$entset") > 0
%%      output '* Do you wish to re-load data into any of your ENTITY SETS?'
%%      output '  Y)es, N)o, A)ll: ';
%%      input <Response>
%%      let <Response> = $tolower ($substring ('#<Response>', 1, 1))
%%
%%      if '#<Response>' <> 'n'
%%         let $setcount = $lastmember ("$entset")
%%         set currentset $entset
%%         top
%%         while $setcount > 0
%%            let <EntName> = $trim(entname)
%%            let <DocName> = $trim ($concat ('_e','',$substring (entname, 1, 16)))
%%            let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%            let <EntName> = $concat("#<DirName>",".","#<EntName>")
%%            let <DocName> = $concat("#<DirName>",".","#<DocName>")
%%            if '#<Response>' <> 'a'
%%               output ''
%%               output '* Do you wish to re-load saved data into the entity set (#<EntName>)'
%%               output '  from document (#<DocName>)?'
%%               output '  Y)es, N)o: ';
%%               input <ReLoadResponse>
%%               let <ReLoadResponse> = $tolower($substring('#<ReLoadResponse>', 1, 1))
%%            else
%%               let <ReLoadResponse> = 'y'
%%            endif

%%            if '#<ReLoadResponse>' <> 'n'
%%               output ''
%%               output 'Re-loading data into entity set (#<EntName>) from document (#<DocName>).'
%%               set membercount on
%%               _access ("#<DirName>")
%%               add #<EntName> from #<DocName>
%%               _release ()
%%               set membercount off
%%            endif

%%            next
%%            let $setcount= $setcount - 1
%%         endwhile
%%      endif
%%   endif

%%   top $relset
%%   compute 1 $relset relfields fields  eval ('')
%%   if $membercount > 0
%%      output ''
%%      output '* Do you wish to re-load data into any of your RELATIONSHIPS with fields?'
%%      output '  Y)es, N)o, A)ll: ';
%%      input <Response>
%%      let <Response> = $tolower ($substring ('#<Response>', 1, 1))
%%
%%      if '#<Response>' <> 'n'
%%         find $relset relfields fields keep relationships ->$relset
%%         if $setcount > 0
%%            while $setcount > 0
%%               let <RelName> = $trim (relname)
%%               let <DocName> = $trim ($concat ('_r', '', $substring(relname, 1, 16)))
%%               let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
%%               let <RelName> = $concat("#<DirName>", ".", "#<RelName>")
%%               let <DocName> = $concat("#<DirName>", ".", "#<DocName>")
%%               if '#<Response>' <> 'a'
%%                   output ''
%%                   output '* Do you wish to re-load saved data into the relationship (#<RelName>)'
%%                   output '  from document (#<DocName>)?'
%%                   output '  Y)es, N)o: ';
%%                   input <ReLoadResponse>
%%                   let <ReLoadResponse> = $tolower ($substring ('#<ReLoadResponse>', 1, 1))
%%               else
%%                   let <ReLoadResponse> = 'y'
%%               endif

%%               if '#<ReLoadResponse>' <> 'n'
%%                   output 'Re-loading data into relationship (#<RelName>) from document (#<DocName>).'
%%                   set membercount on
%%                  _access ("#<DirName>")
%%                   add #<RelName> from #<DocName>
%%                  _release ()
%%                  set membercount off
%%               endif

%%               next $relset
%%               let $setcount = $setcount - 1
%%            endwhile
%%         else
%%            output ''
%%            output 'No relationships with fields found to load.'
%%         endif
%%      endif
%%   endif

   find docs \
      where \
	     ( docs.docname = '_e'? and docs.filename like "sav%ent" ) or \
	     ( docs.docname = '_r'? and docs.filename like "sav%rel" ) \
      -> docsset
   if( $setcount > 0 )
      output 'PERFORMING CLEANUP OF DATA DOCUMENTS.'
   endif
   while $setcount > 0
      let <DocName> = $trim(docname)
      let <DirName> = {$trim(dirname) where dirname <> "", "zim"}
      let <DDDirName> = $trim(dirname)
      %output 'Erase document "#<DDDirName>"?  (y/n) :' ;
      %input <Response>
      let <Response> = 'y'
      if '#<Response>' = 'y'
	  erase doc #<DirName>.#<DocName> from #<DirName>
	  output "#<DirName>.#<DocName>"
	  delete all fields where ownername='#<DocName>' and dirname = "#<DDDirName>"
	  delete docsset
      endif
      next docsset
      let $setcount = $setcount - 1
   endwhile

   SET RESTORE
endprocedure

%----------------------------------------------------------------------
%
% plReleaseDirs
%   Release any dirs accessed during creation.
%
%----------------------------------------------------------------------
localprocedure plReleaseDirs()

   if _diraccess = "STATIC"
      output
      output "RELEASING ALL DIRECTORIES"
      let $setcount = $lastmember("$dirset")
      set currentset $dirset
      bottom $dirset
      while $setcount > 0
         let <DirName> = $trim(dirname)
         let <OwnerDirName> = {$trim(ownerdirname) where Ownerdirname <> "", "zim"}
         output 'RELEASING DIRECTORY #<OwnerDirName>.#<DirName>'
         release #<DirName>
         previous
         let $setcount = $setcount - 1
      endwhile
   endif
   output

endprocedure

%----------------------------------------------------------------------
%
% ZIMLOAD
%   Main program logic.
%
%----------------------------------------------------------------------
%% Exception Handlers

on break
   plBreakHandler()
   if $iswinopen('wDevCen') = $true
      window set input off
      window set output off
   endif
   RETURN
endon

ZOMEnable

let <9> = $MessageBox($concat( \
             'This program should only be used to load a database', \
             '\0dthat was saved with a pre-5.3 version of ZIMSave.\0d', \
             '\0dTo load a database that was saved with a later version,', \
             '\0dcancel this program and use the Import Wizard.\0d'), \
             cInfoMsg, cOKCancelStyle, cDefaultButton2, 'ZimLoad')
if '#<9>' = 'CANCEL'
   RETURN
endif

%----------------------------------------------------------------------
%% Mainline Logic


SET SAVE
set reset
set errors off
set warnings off
set headings off
set checksets off

if $iswinopen('wDevCen') = $true
   window set current wDevCen
   window display
   window set output on
   window set input on
endif

% Access the ZIM utilities directory, $utils
let <UtilsAlreadyAcc> = $false \
    <BreakOut> = ""
plAccessUtilDir()
if( "#<BreakOut>" = "Yes" )
   set exception break
endif

%% Parse the command parameters
plParseCommandLine ($toupper('#<1>'), $toupper('#<2>'), $toupper('#<3>'), \
                    $toupper('#<4>'), $toupper('#<5>'))

if "#<BreakOut>" = "Yes"
   set exception break
endif

%% Now do the LOAD.
if $iswinopen('wDevCen') = $true
   output ''
else
   window clear
endif

%% Explanation of ZIMLoad - do you want to continue?
if "#<UIMode>" = "PROMPT"
   let <Response> = \
          $MessageBox($concat( \
             'This program will load dictionary definitions and', \
             '\0doptionally data from ASCII files created with', \
             '\0dZimSave from a previous release.'), \
             cInfoMsg, cOKCancelStyle, cDefaultButton1, 'ZimSave')
   if '#<Response>' = 'CANCEL'
      SET EXCEPTION BREAK
   endif
endif

%% Load up first batch of Object Dictionary Definitions.
plLoadDDDefs1( "#<LoadFormat>" )

if "#<CrtMode>" <> "NOCREATE"
   %% Create the Objects
   plCrtObjs1()

   %% Load Data into Ents and Rels
   if "#<UIMode>" = "PROMPT"
      let <9> = $MessageBox("Did you save data in _e* or _r* documents ?", \
                            cQuestionMsg, cYesNoStyle, cDefaultButton2, \
                            "ZimLoad")
      if '#<9>' = 'YES'
         plLoadData()
      endif
   endif

   plReleaseDirs()
endif

if "#<UtilsAlreadyAcc>" = $false
   release $utils
endif
if $iswinopen('wDevCen') = $true
   window set input off
   window set output off
endif
SET RESTORE